@@ -1,6 +1,6 @@
 ;;; GDB debugging support for Guile.
 ;;;
-;;; Copyright 2014, 2015, 2017 Free Software Foundation, Inc.
+;;; Copyright 2014, 2015, 2017, 2020 Free Software Foundation, Inc.
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify it
 ;;; under the terms of the GNU General Public License as published by
@@ -49,9 +49,9 @@
 ;; At run time, make sure we load (system base types) from the Guile
 ;; being debugged rather than from the Guile GDB is linked against.
 (set! %load-path
-  (cons "/usr/local/share/guile/2.2" %load-path))
+  (cons "/usr/local/share/guile/3.0" %load-path))
 (set! %load-compiled-path
-  (cons "/usr/local/lib/guile/2.2/site-ccache" %load-compiled-path))
+  (cons "/usr/local/lib/guile/3.0/site-ccache" %load-compiled-path))
 (reload-module (resolve-module '(system base types)))
 
 
@@ -70,12 +70,15 @@ if the information is not available."
 
 (define %gdb-memory-backend
   ;; The GDB back-end to access the inferior's memory.
-  (let ((void* (type-pointer (lookup-type "void"))))
+
+  ;; When run through 'rr replay', even the 'void' type is initially
+  ;; unavailable.  Thus, delay lookup until it's actually needed.
+  (let ((void* (delay (type-pointer (lookup-type "void")))))
     (define (dereference-word address)
       ;; Return the word at ADDRESS.
       (value->integer
        (value-dereference (value-cast (make-value address)
-                                      (type-pointer void*)))))
+                                      (type-pointer (force void*))))))
 
     (define (open address size)
       ;; Return a port to the SIZE bytes starting at ADDRESS.
@@ -167,11 +170,6 @@ if the information is not available."
 ;;; VM stack walking.
 ;;;
 
-(define ip-type (type-pointer (lookup-type "scm_t_uint32")))
-(define fp-type (type-pointer (lookup-type "SCM")))
-(define sp-type (type-pointer (lookup-type "SCM")))
-(define uint-type (type-pointer (lookup-type "scm_t_uintptr")))
-
 (define-record-type <vm-frame>
   (make-vm-frame ip sp fp saved-ip saved-fp)
   vm-frame?
@@ -184,6 +182,9 @@ if the information is not available."
 ;; See libguile/frames.h.
 (define* (vm-frame ip sp fp #:optional (backend %gdb-memory-backend))
   "Return the components of the stack frame at FP."
+  (define ip-type (type-pointer (lookup-type "uint32_t")))
+  (define uint-type (type-pointer (lookup-type "uintptr_t")))
+
   (make-vm-frame ip
                  sp
                  fp
@@ -206,11 +207,12 @@ if the information is not available."
 
 (define (find-vp)
   "Find the scm_vm pointer for the current thread."
-  (let loop ((frame (newest-frame)))
-    (and frame
-         (if (vm-engine-frame? frame)
-             (frame-read-var frame "vp")
-             (loop (frame-older frame))))))
+  (match (lookup-symbol "scm_i_current_thread")
+    ((#f _)
+     #f)
+    ((symbol _)
+     (let ((thread (symbol-value symbol)))
+       (value-field thread "vm")))))
 
 (define (newest-vm-frame)
   "Return the newest VM frame or #f."
@@ -295,6 +297,7 @@ if the information is not available."
       (define (ip-in-symbol? name)
         (let ((sym (lookup-symbol-or-false name)))
           (and sym
+               (not (value-optimized-out? (symbol-value sym)))
                (let* ((val (symbol-value sym))
                       (size (type-sizeof (value-type val)))
                       (char* (type-pointer (arch-char-type (current-arch))))
