@@ -1,6 +1,6 @@
 ;;; Parsing Guile's command-line
 
-;;; Copyright (C) 1994-1998, 2000-2020 Free Software Foundation, Inc.
+;;; Copyright (C) 1994-1998, 2000-2023 Free Software Foundation, Inc.
 
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -40,15 +40,15 @@
             emit-bug-reporting-address))
 
 ;; An initial stab at i18n.
-(define _ gettext)
+(define G_ gettext)
 
 (define *GPLv3+*
-  (_ "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
+  (G_ "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law."))
 
 (define *LGPLv3+*
-  (_ "License LGPLv3+: GNU LGPL 3 or later <http://gnu.org/licenses/lgpl.html>.
+  (G_ "License LGPLv3+: GNU LGPL 3 or later <http://gnu.org/licenses/lgpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law."))
 
@@ -66,7 +66,7 @@ There is NO WARRANTY, to the extent permitted by law."))
 (define* (version-etc package version #:key
                       (port (current-output-port))
                       ;; FIXME: authors
-                      (copyright-year 2020)
+                      (copyright-year 2023)
                       (copyright-holder "Free Software Foundation, Inc.")
                       (copyright (format #f "Copyright (C) ~a ~a"
                                          copyright-year copyright-holder))
@@ -79,8 +79,8 @@ There is NO WARRANTY, to the extent permitted by law."))
 
   (if packager
       (if packager-version
-          (format port (_ "Packaged by ~a (~a)\n") packager packager-version)
-          (format port (_ "Packaged by ~a\n") packager)))
+          (format port (G_ "Packaged by ~a (~a)\n") packager packager-version)
+          (format port (G_ "Packaged by ~a\n") packager)))
   
   (display copyright port)
   (newline port)
@@ -98,15 +98,15 @@ There is NO WARRANTY, to the extent permitted by law."))
                                            package
                                            "/"))
                                      packager packager-bug-address)
-  (format port (_ "\nReport bugs to: ~a\n") bug-address)
+  (format port (G_ "\nReport bugs to: ~a\n") bug-address)
   (if (and packager packager-bug-address)
-      (format port (_ "Report ~a bugs to: ~a\n") packager packager-bug-address))
-  (format port (_ "~a home page: <~a>\n") package url)
+      (format port (G_ "Report ~a bugs to: ~a\n") packager packager-bug-address))
+  (format port (G_ "~a home page: <~a>\n") package url)
   (format port
-          (_ "General help using GNU software: <http://www.gnu.org/gethelp/>\n")))
+          (G_ "General help using GNU software: <http://www.gnu.org/gethelp/>\n")))
 
 (define *usage*
-  (_ "Evaluate code with Guile, interactively or from a script.
+  (G_ "Evaluate code with Guile, interactively or from a script.
 
   [-s] FILE      load source code from FILE, and exit
   -c EXPR        evalute expression EXPR, and exit
@@ -138,6 +138,10 @@ If FILE begins with `-' the -s switch is mandatory.
   -q             inhibit loading of user init file
   --use-srfi=LS  load SRFI modules for the SRFIs in LS,
                  which is a list of numbers like \"2,13,14\"
+  --r6rs         change initial Guile environment to better support
+                 R6RS
+  --r7rs         change initial Guile environment to better support
+                 R7RS
   -h, --help     display this help and exit
   -v, --version  display version information and exit
   \\              read arguments from following script lines"))
@@ -151,7 +155,7 @@ If FILE begins with `-' the -s switch is mandatory.
       (apply format port fmt args)
       (newline port))
 
-    (format port (_ "Usage: ~a [OPTION]... [FILE]...\n") name)
+    (format port (G_ "Usage: ~a [OPTION]... [FILE]...\n") name)
     (display *usage* port)
     (newline port)
 
@@ -370,6 +374,14 @@ If FILE begins with `-' the -s switch is mandatory.
               (parse args
                      (cons `(use-srfis ',srfis) out))))
 
+           ((string=? "--r6rs" arg)
+            (parse args
+                   (cons '(install-r6rs!) out)))
+
+           ((string=? "--r7rs" arg)
+            (parse args
+                   (cons '(install-r7rs!) out)))
+
            ((string=? arg "--listen")   ; start a repl server
             (parse args
                    (cons '((@@ (system repl server) spawn-server)) out)))
