@@ -1,6 +1,6 @@
 ;;; Effects analysis on Tree-IL
 
-;; Copyright (C) 2011, 2012, 2013 Free Software Foundation, Inc.
+;; Copyright (C) 2011, 2012, 2013, 2021 Free Software Foundation, Inc.
 
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -294,10 +294,10 @@ of an expression."
            (logior (cause &toplevel)
                    (cause &type-check)
                    (compute-effects exp)))
-          (($ <toplevel-define> _ name exp)
+          (($ <toplevel-define> _ _ name exp)
            (logior (cause &toplevel)
                    (compute-effects exp)))
-          (($ <toplevel-set> _ name exp)
+          (($ <toplevel-set> _ _ name exp)
            (logior (cause &toplevel)
                    (compute-effects exp)))
           (($ <primitive-ref>)
@@ -416,6 +416,15 @@ of an expression."
                    (cause &type-check)
                    (cause &variable)))
 
+          (($ <primcall> _ '%variable-ref (v))
+           (logior (compute-effects v)
+                   (cause &type-check) ;; For the unbound check.
+                   &variable))
+          (($ <primcall> _ '%variable-set! (v x))
+           (logior (compute-effects v)
+                   (compute-effects x)
+                   (cause &variable)))
+
           (($ <primcall> _ 'struct-ref (s n))
            (logior (compute-effects s)
                    (compute-effects n)
@@ -579,8 +588,12 @@ of an expression."
 
           (($ <prompt> _ escape-only? tag body handler)
            (logior (compute-effects tag)
-                   (compute-effects body)
-                   (compute-effects handler)))
+                   (compute-effects (if escape-only?
+                                        body
+                                        (make-call #f body '())))
+                   ;; Calls handler with probably wrong argument count,
+                   ;; but that will just add a &type-check effect.
+                   (compute-effects (make-call #f handler '()))))
 
           (($ <abort> _ tag args tail)
            (logior &all-effects-but-bailout
