@@ -1,5 +1,5 @@
 ;;; 'SCM' type tag decoding.
-;;; Copyright (C) 2014, 2015, 2017, 2018 Free Software Foundation, Inc.
+;;; Copyright (C) 2014, 2015, 2017, 2018, 2022 Free Software Foundation, Inc.
 ;;;
 ;;; This library is free software; you can redistribute it and/or modify it
 ;;; under the terms of the GNU Lesser General Public License as published by
@@ -28,6 +28,7 @@
   #:use-module (ice-9 format)
   #:use-module (ice-9 vlist)
   #:use-module (system foreign)
+  #:use-module (system base types internal)
   #:export (%word-size
 
             memory-backend
@@ -178,6 +179,10 @@ return the corresponding string."
                  (a (logand bits (bitwise-not n))))
              consequent)
            alternate)))
+    ((match-bit-pattern bits (= c) consequent alternate)
+     (if (= bits c)
+         consequent
+         alternate))
     ((match-bit-pattern bits (x & n = c) consequent alternate)
      (let ((tag (logand bits n)))
        (if (= tag c)
@@ -242,53 +247,6 @@ the matching bits, possibly with bitwise operations to extract it from BITS."
        (match-scm-clauses bits* clauses ...)))))
 
 
-;;;
-;;; Tags---keep in sync with libguile/tags.h!
-;;;
-
-;; Immediate values.
-(define %tc2-int 2)
-(define %tc3-imm24 4)
-
-(define %tc3-cons 0)
-(define %tc3-int1 %tc2-int)
-(define %tc3-int2 (+ %tc2-int 4))
-
-(define %tc8-char (+ 8 %tc3-imm24))
-(define %tc8-flag (+ %tc3-imm24 0))
-
-;; Cell types.
-(define %tc3-struct #x01)
-(define %tc7-symbol #x05)
-(define %tc7-variable #x07)
-(define %tc7-vector #x0d)
-(define %tc7-wvect #x0f)
-(define %tc7-string #x15)
-(define %tc7-number #x17)
-(define %tc7-hashtable #x1d)
-(define %tc7-pointer #x1f)
-(define %tc7-fluid #x25)
-(define %tc7-stringbuf #x27)
-(define %tc7-dynamic-state #x2d)
-(define %tc7-frame #x2f)
-(define %tc7-keyword #x35)
-(define %tc7-syntax #x3d)
-(define %tc7-program #x45)
-(define %tc7-vm-continuation #x47)
-(define %tc7-bytevector #x4d)
-(define %tc7-weak-set #x55)
-(define %tc7-weak-table #x57)
-(define %tc7-array #x5d)
-(define %tc7-bitvector #x5f)
-(define %tc7-port #x7d)
-(define %tc7-smob #x77)
-
-(define %tc16-bignum (+ %tc7-number (* 1 256)))
-(define %tc16-real (+ %tc7-number (* 2 256)))
-(define %tc16-complex (+ %tc7-number (* 3 256)))
-(define %tc16-fraction (+ %tc7-number (* 4 256)))
-
-
 ;; "Stringbufs".
 (define-record-type <stringbuf>
   (stringbuf string)
@@ -395,13 +353,14 @@ TYPE-NUMBER."
                                                  (%visited-cells))))
        body ...))))
 
-(define (address->inferior-struct address vtable-data-address backend)
+(define (address->inferior-struct address vtable-address backend)
   "Read the struct at ADDRESS using BACKEND.  Return an 'inferior-struct'
 object representing it."
-  (define %vtable-layout-index 0)
-  (define %vtable-name-index 5)
+  (define %vtable-layout-index vtable-index-layout)
+  (define %vtable-name-index 4)
 
-  (let* ((layout-address (+ vtable-data-address
+  (let* ((vtable-data-address (+ vtable-address %word-size))
+         (layout-address (+ vtable-data-address
                             (* %vtable-layout-index %word-size)))
          (layout-bits    (dereference-word backend layout-address))
          (layout         (scm->object layout-bits backend))
@@ -412,7 +371,7 @@ object representing it."
     (if (symbol? layout)
         (let* ((layout (symbol->string layout))
                (len    (/ (string-length layout) 2))
-               (slots  (dereference-word backend (+ address %word-size)))
+               (slots  (+ address %word-size))
                (port   (memory-port backend slots (* len %word-size)))
                (fields (get-bytevector-n port (* len %word-size)))
                (result (inferior-struct name #f)))
@@ -434,11 +393,11 @@ using BACKEND."
   (or (and=> (vhash-assv address (%visited-cells)) cdr) ; circular object
       (let ((port (memory-port backend address)))
         (match-cell port
-          (((vtable-data-address & 7 = %tc3-struct))
+          (((vtable-address & 7 = %tc3-struct))
            (address->inferior-struct address
-                                     (- vtable-data-address %tc3-struct)
+                                     (- vtable-address %tc3-struct)
                                      backend))
-          (((_ & #x7f = %tc7-symbol) buf hash props)
+          (((_ & #x7f = %tc7-symbol) buf hash)
            (match (cell->object buf backend)
              (($ <stringbuf> string)
               (string->symbol string))))
@@ -471,7 +430,7 @@ using BACKEND."
                           (bytevector->uint-list words (native-endianness)
                                                  %word-size)))
                vector)))
-          (((_ & #x7f = %tc7-wvect))
+          (((_ & #x7f = %tc7-weak-vector))
            (inferior-object 'weak-vector address))   ; TODO: show elements
           (((_ & #x7f = %tc7-fluid) init-value)
            (inferior-object 'fluid address))
@@ -483,14 +442,14 @@ using BACKEND."
            (inferior-object 'program address))
           (((_ & #xffff = %tc16-bignum))
            (inferior-object 'bignum address))
-          (((_ & #xffff = %tc16-real) pad)
-           (let* ((address (+ address (* 2 %word-size)))
+          (((_ & #xffff = %tc16-flonum) pad)
+           (let* ((address (+ address (match %word-size (4 8) (8 8))))
                   (port    (memory-port backend address (sizeof double)))
                   (words   (get-bytevector-n port (sizeof double))))
              (bytevector-ieee-double-ref words 0 (native-endianness))))
-          (((_ & #x7f = %tc7-number) mpi)
+          (((_ & #x7f = %tc7-heap-number) mpi)
            (inferior-object 'number address))
-          (((_ & #x7f = %tc7-hashtable) buckets meta-data unused)
+          (((_ & #x7f = %tc7-hash-table) buckets meta-data unused)
            (inferior-object 'hash-table address))
           (((_ & #x7f = %tc7-pointer) address)
            (make-pointer address))
@@ -522,11 +481,11 @@ using BACKEND."
   "Return the Scheme object corresponding to BITS, the bits of an 'SCM'
 object."
   (match-scm bits
-    (((integer << 2) || %tc2-int)
+    (((integer << 2) || %tc2-fixnum)
      integer)
-    ((address & 6 = %tc3-cons)
+    ((address & 7 = %tc3-heap-object)
      (let* ((type  (dereference-word backend address))
-            (pair? (not (bit-set? 0 type))))
+            (pair? (= (logand type #b1) %tc1-pair)))
        (if pair?
            (or (and=> (vhash-assv address (%visited-cells)) cdr)
                (let ((car    type)
@@ -541,16 +500,13 @@ object."
            (cell->object address backend))))
     (((char << 8) || %tc8-char)
      (integer->char char))
-    (((flag << 8) || %tc8-flag)
-     (case flag
-       ((0)  #f)
-       ((1)  #nil)
-       ((3)  '())
-       ((4)  #t)
-       ((8)  (if #f #f))
-       ((9)  (inferior-object 'undefined bits))
-       ((10) (eof-object))
-       ((11) (inferior-object 'unbound bits))))))
+    ((= %tc16-false) #f)
+    ((= %tc16-nil) #nil)
+    ((= %tc16-null) '())
+    ((= %tc16-true) #t)
+    ((= %tc16-unspecified) (if #f #f))
+    ((= %tc16-undefined) (inferior-object 'undefined bits))
+    ((= %tc16-eof) (eof-object))))
 
 ;;; Local Variables:
 ;;; eval: (put 'match-scm 'scheme-indent-function 1)
